<?php
/**
 * User-related Functions
 *
 * @package BadgeOS
 * @subpackage Admin
 * @author LearningTimes, LLC
 * @license http://www.gnu.org/licenses/agpl.txt GNU AGPL v3.0
 * @link https://credly.com
 */

/**
 * Get a user's badgeos achievements
 *
 * @since  1.0.0
 * @param  array $args An array of all our relevant arguments
 * @return array       An array of all the achievement objects that matched our parameters, or empty if none
 */
function badgeos_get_user_achievements( $args = array() ) {

	// Setup our default args
	$defaults = array(
		'user_id'          => 0,     // The given user's ID
		'site_id'          => get_current_blog_id(), // The given site's ID
		'achievement_id'   => false, // A specific achievement's post ID
		'achievement_type' => false, // A specific achievement type
		'since'            => 0,     // A specific timestamp to use in place of $limit_in_days
	);
	$args = wp_parse_args( $args, $defaults );

	// Use current user's ID if none specified
	if ( ! $args['user_id'] )
		$args['user_id'] = get_current_user_id();

	// Grab the user's current achievements
	$achievements = ( $earned_items = get_user_meta( absint( $args['user_id'] ), '_badgeos_achievements', true ) ) ? (array) $earned_items : array();

	// If we want all sites (or no specific site), return the full array
	if ( empty( $achievements ) || empty( $args['site_id']) || 'all' == $args['site_id'] )
		return $achievements;

	if ( !array_key_exists( $args['site_id'], $achievements ) ) { return; }
	
	// Otherwise, we only want the specific site's achievements
	$achievements = $achievements[$args['site_id']];

	if ( is_array( $achievements) && ! empty( $achievements ) ) {
		foreach ( $achievements as $key => $achievement ) {

			// Drop any achievements earned before our since timestamp
			if ( absint($args['since']) > $achievement->date_earned )
				unset($achievements[$key]);

			// Drop any achievements that don't match our achievement ID
			if ( ! empty( $args['achievement_id'] ) && absint( $args['achievement_id'] ) != $achievement->ID )
				unset($achievements[$key]);

			// Drop any achievements that don't match our achievement type
			if ( ! empty( $args['achievement_type'] ) && ( $args['achievement_type'] != $achievement->post_type && ( !is_array( $args['achievement_type'] ) || !in_array( $achievement->post_type, $args['achievement_type'] ) ) ) )
				unset($achievements[$key]);

			//unset hidden achievements
			$hidden = badgeos_get_hidden_achievement_by_id( $achievement->ID );
			if( !empty( $hidden ) && isset($args['display']))
				unset($achievements[$key]);

		}
	}

	// Return our $achievements array_values (so our array keys start back at 0), or an empty array
	return ( is_array( $achievements ) ? array_values( $achievements ) : array());

}

/**
 * Updates the user's earned achievements
 *
 * We can either replace the achievement's array, or append new achievements to it.
 *
 * @since  1.0.0
 * @param  array        $args An array containing all our relevant arguments
 * @return integer|bool       The updated umeta ID on success, false on failure
 */
function badgeos_update_user_achievements( $args = array() ) {

	// Setup our default args
	$defaults = array(
		'user_id'          => 0,     // The given user's ID
		'site_id'          => get_current_blog_id(), // The given site's ID
		'all_achievements' => false, // An array of ALL achievements earned by the user
		'new_achievements' => false, // An array of NEW achievements earned by the user
	);
	$args = wp_parse_args( $args, $defaults );

	// Use current user's ID if none specified
	if ( ! $args['user_id'] )
		$args['user_id'] = wp_get_current_user()->ID;

	// Grab our user's achievements
	$achievements = badgeos_get_user_achievements( array( 'user_id' => absint( $args['user_id'] ), 'site_id' => 'all' ) );

	// If we don't already have an array stored for this site, create a fresh one
	if ( !isset( $achievements[$args['site_id']] ) )
		$achievements[$args['site_id']] = array();

	// Determine if we should be replacing or appending to our achievements array
	if ( is_array( $args['all_achievements'] ) )
		$achievements[$args['site_id']] = $args['all_achievements'];
	elseif ( is_array( $args['new_achievements'] ) && ! empty( $args['new_achievements'] ) )
		$achievements[$args['site_id']] = array_merge( $achievements[$args['site_id']], $args['new_achievements'] );

	// Finally, update our user meta
	return update_user_meta( absint( $args['user_id'] ), '_badgeos_achievements', $achievements);

}

/**
 * Display achievements for a user on their profile screen
 *
 * @since  1.0.0
 * @param  object $user The current user's $user object
 * @return void
 */
function badgeos_user_profile_data( $user = null ) {

	$achievement_ids = array();

    echo '<h2>' . __( 'BadgeOS Email Notifications', 'badgeos' ) . '</h2>';
    echo '<table class="form-table">';
    echo '<tr>';
        echo '<th scope="row">' . __( 'Email Preference', 'badgeos' ) . '</th>';
        echo '<td>';
            echo '<label for="_badgeos_can_notify_user"><input type="checkbox" name="_badgeos_can_notify_user" id="_badgeos_can_notify_user" value="1" ' . checked( badgeos_can_notify_user( $user->ID ), true, false ) . '/>' . __( 'Enable BadgeOS Email Notifications', 'badgeos' ) . '</label>';
        echo '</td>';
    echo '</tr>';
    echo '</table>';

    echo '<input id="badgeos_user_id" name="badgeos_user_id" value="'.$user->ID.'" type="hidden" />';

	//verify uesr meets minimum role to view earned badges
	if ( current_user_can( badgeos_get_manager_capability() ) ) {

	    $achievements = badgeos_get_user_achievements( array( 'user_id' => absint( $user->ID ) ) );

		echo '<h2>' . __( 'Earned Achievements', 'badgeos' ) . '</h2>';

		echo '<table class="form-table">';
		echo '<tr>';
			echo '<th><label for="user_points">' . __( 'Earned Points', 'badgeos' ) . '</label></th>';
			echo '<td>';
				echo '<input type="text" name="user_points" id="user_points" value="' . badgeos_get_users_points( $user->ID ) . '" class="regular-text" /><br />';
				echo '<span class="description">' . __( "The user's points total. Entering a new total will automatically log the change and difference between totals.", 'badgeos' ) . '</span>';
			echo '</td>';
		echo '</tr>';

		echo '<tr><td colspan="2">';
		// List all of a user's earned achievements
		if ( $achievements ) {
			echo '<table class="widefat badgeos-table">';
			echo '<thead><tr>';
            echo '<th width="5%"><input type="checkbox" id="badgeos_ach_check_all" name="badgeos_ach_check_all" /></th>';
				echo '<th>'. __( 'Image', 'badgeos' ) .'</th>';
				echo '<th>'. __( 'Name', 'badgeos' ) .'</th>';
				echo '<th>'. __( 'Points', 'badgeos' ) .'</th>';
			echo '</tr></thead>';

            $ach_index = 0;
            $achievement_exists = false;

			foreach ( $achievements as $achievement ) {

                if( $achievement->post_type != 'step' ) {
                    $achievement_exists = true;
                    $revoke_url = add_query_arg( array(
                        'action'         => 'revoke',
                        'user_id'        => absint( $user->ID ),
                        'achievement_id' => absint( $achievement->ID ),
                    ) );


                    echo '<tr>';
                    echo '<td width="5%" style="text-align:center;"><input type="checkbox" id="badgeos_ach_check_indi_'.$achievement->ID.'" value="'.$achievement->ID.'_'.$ach_index.'" name="badgeos_ach_check_indis[]" /></td>';
                    echo '<td width="20%">'. badgeos_get_achievement_post_thumbnail( $achievement->ID, array( 50, 50 ) ) .'</td>';
                    echo '<td width="55%">';
                    $achievement_title = get_the_title( $achievement->ID );
                    if( empty( $achievement_title ) ) {
                        echo '<a class="post-edit-link" href="javascript:;">'. esc_html( $achievement->title ) .'</a>';
                    } else {
                        $achievement_title = edit_post_link( $achievement_title, '', '', $achievement->ID );
                    }
                    echo '</td>';

                    echo '<td width="20%">'.intval( $achievement->points ).' </td>';
                    echo '</tr>';
                    $ach_index += 1;
                    $achievement_ids[] = $achievement->ID;
                }

            }

            if( $achievement_exists ) {
                echo '<tr>';
                echo '<td colspan="5" class="bulk-delete-detail">'.__( 'Select achievement(s) before clicking on revoke selected button', 'badgeos' ).'</td>';
                echo '</tr>';
                echo '<tr>';
                echo '<td colspan="5" class="bulk-delete-detail"><button type="button" id="badgeos_btn_revoke_bulk_achievements" class="button button-primary">'.__( 'Revoke Selected', 'badgeos' ).'</button><img id="revoke-badges-loader" src="'. admin_url( '/images/spinner-2x.gif' ) .'" /></td>';
                echo '</tr>';
            } else {
			    echo '<tr>';
			    echo '<td>'. __( 'No Achievement Found.', 'badgeos' ) .'</td>';
			    echo '</tr>';
            }

            echo '</table>';
		}

		echo '</td></tr>';
		echo '</table>';

		// If debug mode is on, output our achievements array
		if ( badgeos_is_debug_mode() ) {

			echo __( 'DEBUG MODE ENABLED', 'badgeos' ) . '<br />';
			echo __( 'Metadata value for:', 'badgeos' ) . ' _badgeos_achievements<br />';

			var_dump ( $achievements );

		}

		echo '<br/>';

		// Output markup for awarding achievement for user
		badgeos_profile_award_achievement( $user, $achievement_ids );

	}

}
add_action( 'show_user_profile', 'badgeos_user_profile_data' );
add_action( 'edit_user_profile', 'badgeos_user_profile_data' );

/**
 * revokes achivements
 *
 * @since  1.0.0
 * @param  int  $user_id      User ID being saved
 * @return none
 */
function delete_badgeos_bulk_achievements_records( ){

    $user_recs = $_POST['achievements'];
    $user_id = $_POST['user_id'];
    if( is_array( $user_recs ) && count( $user_recs ) > 0 ) {
        $achievements = array();
        $indexes = array();
        foreach( $user_recs as $rec ) {

            $params = explode("_", $rec);

            $achievements[] = $params[0];
            $indexes[] = $params[1];
        }
        $my_achievements = badgeos_get_user_achievements( array( 'user_id' => $user_id ) );

        $index = 0;
        $new_achievements = array();
        $delete_achievement = array();
        foreach( $my_achievements as $my_achs ) {
            if( $my_achs->post_type != 'step' ) {
                if( in_array( $index, $indexes ) && in_array( $my_achs->ID, $achievements ) ) {
                    $delete_achievement[] = $my_achs->ID;
                } else {
                    $new_achievements[] = $my_achs;
                }
                $index += 1;
            } else {
                $new_achievements[] = $my_achs;
            }
        }

        foreach( $delete_achievement as $del_ach_id ) {
            $children = badgeos_get_achievements( array( 'children_of' => $del_ach_id) );
            foreach( $children as $child ) {
                foreach( $new_achievements as $index => $item ) {

                    if( $child->ID == $item->ID ) {
                        unset( $new_achievements[ $index ] );
                        $new_achievements = array_values( $new_achievements );
                        badgeos_decrement_user_trigger_count( $user_id, $child->ID, $del_ach_id );
                        break;
                    }
                }
            }
        }
        $new_achievements = array_values( $new_achievements );


        // Update user's earned achievements
        badgeos_update_user_achievements( array( 'user_id' => $user_id, 'all_achievements' => $new_achievements ) );

        // Available hook for taking further action when an achievement is revoked
        do_action( 'badgeos_revoke_bulk_achievement', $user_id, $delete_achievement );
        echo 'success';
    } else {
        echo __( 'Please, select some achievement(s) to delete', 'badgeos' );
    }
    exit;
}
add_action( 'wp_ajax_delete_badgeos_bulk_achievements', 'delete_badgeos_bulk_achievements_records' );

/**
 * Save extra user meta fields to the Edit Profile screen
 *
 * @since  1.0.0
 * @param  int  $user_id      User ID being saved
 * @return mixed			  false if current user can not edit users, void if can
 */
function badgeos_save_user_profile_fields( $user_id = 0 ) {

	if ( ! current_user_can( 'edit_user', $user_id ) ) {
		return false;
	}

	$can_notify = isset( $_POST['_badgeos_can_notify_user'] ) ? 'true' : 'false';
	update_user_meta( $user_id, '_badgeos_can_notify_user', $can_notify );

	// Update our user's points total, but only if edited
	if ( isset( $_POST['user_points'] ) && $_POST['user_points'] != badgeos_get_users_points( $user_id ) ) {
		badgeos_update_users_points( $user_id, intval( $_POST['user_points'] ), get_current_user_id() );
	}

}
add_action( 'personal_options_update', 'badgeos_save_user_profile_fields' );
add_action( 'edit_user_profile_update', 'badgeos_save_user_profile_fields' );

/**
 * Generate markup for awarding an achievement to a user
 *
 * @since  1.0.0
 * @param  object $user         The current user's $user object
 * @param  array  $achievements array of user-earned achievement IDs
 * @return string               concatenated markup
 */
function badgeos_profile_award_achievement( $user = null, $achievement_ids = array() ) {

	// Grab our achivement types
	$achievement_types = badgeos_get_achievement_types();
	?>

	<h2><?php _e( 'Award an Achievement', 'badgeos' ); ?></h2>

	<table class="form-table">
		<tr>
			<th><label for="thechoices"><?php _e( 'Select an Achievement Type to Award:', 'badgeos' ); ?></label></th>
			<td>
				<select id="thechoices">
				<option></option>
				<?php
				foreach ( $achievement_types as $achievement_slug => $achievement_type ) {
                    if( $achievement_slug != 'step' ) {
                        echo '<option value="'. $achievement_slug .'">' . ucwords( $achievement_type['single_name'] ) .'</option>';
                    }
                }
				?>
				</select>
			</td>
		</tr>
		<tr><td id="boxes" colspan="2">
			<?php foreach ( $achievement_types as $achievement_slug => $achievement_type ) : ?>
                <?php if( $achievement_slug != 'step' ) { ?>
				<table id="<?php echo esc_attr( $achievement_slug ); ?>" class="widefat badgeos-table">
                    <thead>
                    <tr>
                        <th><?php _e( 'Image', 'badgeos' ); ?></th>
                        <th><?php echo ucwords( $achievement_type['single_name'] ); ?></th>
                        <th><?php _e( 'Action', 'badgeos' ); ?></th>
                    </tr>
                    </thead>

                    <tbody>
					<?php
					// Load achievement type entries
					$the_query = new WP_Query( array(
						'post_type'      => $achievement_slug,
						'posts_per_page' => '999',
						'post_status'    => 'publish'
					) );

					if ( $the_query->have_posts() ) : ?>

						<?php while ( $the_query->have_posts() ) : $the_query->the_post();

							// Setup our award URL
							$award_url = add_query_arg( array(
								'action'         => 'award',
								'achievement_id' => absint( get_the_ID() ),
								'user_id'        => absint( $user->ID )
							) );
							?>
							<tr>
								<td><?php the_post_thumbnail( array( 50, 50 ) ); ?></td>
								<td>
									<?php echo edit_post_link( get_the_title() ); ?>
								</td>
								<td>
									<a href="<?php echo esc_url( wp_nonce_url( $award_url, 'badgeos_award_achievement' ) ); ?>"><?php printf( __( 'Award %s', 'badgeos' ), ucwords( $achievement_type['single_name'] ) ); ?></a>
    <!-- <?php if ( in_array( get_the_ID(), (array) $achievement_ids ) ) :
										// Setup our revoke URL
										$revoke_url = add_query_arg( array(
											'action'         => 'revoke',
											'user_id'        => absint( $user->ID ),
											'achievement_id' => absint( get_the_ID() ),
										) );
										?>
										<span class="delete"><a class="error" href="<?php echo esc_url( wp_nonce_url( $revoke_url, 'badgeos_revoke_achievement' ) ); ?>"><?php _e( 'Revoke Award', 'badgeos' ); ?></a></span>
									<?php endif; ?> -->

								</td>
							</tr>
						<?php endwhile; ?>

					<?php else : ?>
						<tr>
							<th><?php printf( __( 'No %s found.', 'badgeos' ), $achievement_type['plural_name'] ); ?></th>
						</tr>
					<?php endif; wp_reset_postdata(); ?>

					</tbody>
					</table><!-- #<?php echo esc_attr( $achievement_slug ); ?> -->
                <?php } ?>
			<?php endforeach; ?>
		</td><!-- #boxes --></tr>
	</table>

	<script type="text/javascript">
		jQuery(document).ready(function($){
			<?php foreach ( $achievement_types as $achievement_slug => $achievement_type ) { ?>
				$('#<?php echo $achievement_slug; ?>').hide();
			<?php } ?>
			$("#thechoices").change(function(){
				if ( 'all' == this.value )
					$("#boxes").children().show();
				else
					$("#" + this.value).show().siblings().hide();
			}).change();
		});
	</script>
	<?php
}

/**
 * Process the adding/revoking of achievements on the user profile page
 *
 * @since  1.0.0
 * @return void
 */
function badgeos_process_user_data() {

	//verify uesr meets minimum role to view earned badges
	if ( current_user_can( badgeos_get_manager_capability() ) ) {

		// Process awarding achievement to user
		if ( isset( $_GET['action'] ) && 'award' == $_GET['action'] &&  isset( $_GET['user_id'] ) && isset( $_GET['achievement_id'] ) ) {

			// Verify our nonce
			check_admin_referer( 'badgeos_award_achievement' );

			// Award the achievement
			badgeos_award_achievement_to_user( absint( $_GET['achievement_id'] ), absint( $_GET['user_id'] ) );

			// Redirect back to the user editor
			wp_redirect( add_query_arg( 'user_id', absint( $_GET['user_id'] ), admin_url( 'user-edit.php' ) ) );
			exit();
		}

		// Process revoking achievement from a user
		if ( isset( $_GET['action'] ) && 'revoke' == $_GET['action'] && isset( $_GET['user_id'] ) && isset( $_GET['achievement_id'] ) ) {

			// Verify our nonce
			check_admin_referer( 'badgeos_revoke_achievement' );

			// Revoke the achievement
			badgeos_revoke_achievement_from_user( absint( $_GET['achievement_id'] ), absint( $_GET['user_id'] ) );

			// Redirect back to the user editor
			wp_redirect( add_query_arg( 'user_id', absint( $_GET['user_id'] ), admin_url( 'user-edit.php' ) ) );
			exit();

		}

	}

}
add_action( 'init', 'badgeos_process_user_data' );

/**
 * Returns array of achievement types a user has earned across a multisite network
 *
 * @since  1.2.0
 * @param  integer $user_id  The user's ID
 * @return array             An array of post types
 */
function badgeos_get_network_achievement_types_for_user( $user_id ) {
	global $blog_id;

	// Store a copy of the original ID for later
	$cached_id = $blog_id;

	// Assume we have no achievement types
	$all_achievement_types = array();

	// Loop through all active sites
	$sites = badgeos_get_network_site_ids();
	foreach( $sites as $site_blog_id ) {

		// If we're polling a different blog, switch to it
		if ( $blog_id != $site_blog_id ) {
			switch_to_blog( $site_blog_id );
		}

		// Merge earned achievements to our achievement type array
		$achievement_types = badgeos_get_user_earned_achievement_types( $user_id );
		if ( is_array($achievement_types) ) {
			$all_achievement_types = array_merge($achievement_types,$all_achievement_types);
		}
	}

	if ( is_multisite() ) {
		// Restore the original blog so the sky doesn't fall
		switch_to_blog( $cached_id );
	}

	// Pare down achievement type list so we return no duplicates
	$achievement_types = array_unique( $all_achievement_types );

	// Return all found achievements
	return $achievement_types;
}

/**
 * Check if a user has disabled email notifications.
 *
 * @since  1.4.0
 *
 * @param  int   $user_id User ID.
 * @return bool           True if user can be emailed, otherwise false.
 */
function badgeos_can_notify_user( $user_id = 0 ) {
	if ( empty( $user_id ) ) {
		$user_id = get_current_user_id();
	}

	return 'false' !== get_user_meta( $user_id, '_badgeos_can_notify_user', true );
}
